'==============================================================================
' File: 3d-DEMO.BAS
' Desc: Program to demonstrate some of the PB-XLIB functions
'------------------------------------------------------------------------------

$DIM      ALL           '
$LIB      ALL OFF       '
$ERROR    ALL OFF       '
$STACK    4096          '
$COMPILE  EXE           '
$OPTIMIZE SIZE          ' Optimize for smaller code
$DYNAMIC

DEFINT    A-Z           '

'------------------------------------------------------------------------------

$INCLUDE "xlib.pbi"
$LINK    "xlib.pbl"

'------------------------------------------------------------------------------

'-- Types -------------------------------------------------

TYPE Amp
  CurrentX  AS INTEGER
  CurrentY  AS INTEGER
  StartX    AS INTEGER
  StartY    AS INTEGER
  EndX      AS INTEGER
  EndY      AS INTEGER
  Stage     AS STRING * 1
  Direction AS INTEGER
END TYPE

'==============================================================================

DECLARE SUB Initialize()
DECLARE SUB GetCharacter()
DECLARE SUB GetMode()
DECLARE SUB Main()
DECLARE SUB DrawMan()
DECLARE SUB DrawScreen()
DECLARE SUB Keystroke (First, Second)
DECLARE SUB xGet(ByVal X1, ByVal Y1, ByVal X2, ByVal Y2, string)

'==============================================================================

DIM vScreen AS SHARED STRING,_
    Blank   AS SHARED STRING,_
    Active  AS SHARED STRING,_
    Pal     AS SHARED STRING,_
    Walls   AS SHARED STRING,_
    Figure  AS SHARED STRING,_
    Man     AS SHARED Amp,_
    zMap(30)  AS SHARED STRING

DIM I       AS SHARED STRING,_
    Char    AS SHARED STRING,  Mode   AS SHARED STRING,_
    First   AS SHARED INTEGER, Second AS SHARED INTEGER,_
    X       AS SHARED INTEGER, Y      AS SHARED INTEGER

'==============================================================================

Initialize
Main

'==============================================================================

'----------------------------------------------------------
'- Initialize
'----------------------------------------------------------

SUB Initialize

  GetCharacter
  GetMode

  '-- Load the sprites for this animation -------

  LoadBsFile "WALLS.DAT", Walls      ' Walls
  LoadBsFile Char$ + ".DAT", Figure  ' Figures

  '-- Load the palette for this animation -------

  LoadBsPal "SCROLL.PLT", Pal

  '-- Set intial paramters ----------------------

  Man.CurrentX  = 3
  Man.CurrentY  = 4
  Man.StartX    = 1
  Man.StartY    = 1
  Man.EndX      = 9
  Man.EndY      = 9
  Man.Stage     = "1"
  Man.Direction = 2

  '-- The Map -----------------------------------

  zMap(1)  = " Ŀ        Ŀ       Ŀ "
  zMap(2)  = " Ŀ          "
  zMap(3)  = "           "
  zMap(4)  = " Ŀ  Ŀ        "
  zMap(5)  = "             "
  zMap(6)  = "             "
  zMap(7)  = " Ŀ     Ŀ "
  zMap(8)  = "   Ŀ      Ŀ                 "
  zMap(9)  = "              Ŀ "
  zMap(10) = "               "
  zMap(11) = " Ŀ      Ŀ        "
  zMap(12) = "   Ŀ                    "
  zMap(13) = "  "
  zMap(14) = "  "
  zMap(15) = " Ŀ "
  zMap(16) = "         Ŀ            "
  zMap(17) = " Ŀ              "
  zMap(18) = "               "
  zMap(19) = " Ŀ  Ŀ            "
  zMap(20) = "                 "
  zMap(21) = "                 "
  zMap(22) = " Ŀ     Ŀ     "
  zMap(23) = "   Ŀ      Ŀ                 "
  zMap(24) = "                        "
  zMap(25) = "                        "
  zMap(26) = " Ŀ      Ŀ                 "
  zMap(27) = "   Ŀ                             "
  zMap(28) = "                     "
  zMap(29) = "                     "
  zMap(30) = "                     "

  '-- Set Screen mode and palette ---------------

  SELECT CASE VAL(Mode$)
    CASE 1 : xSetModeX %xMode320x200
    CASE 2 : xSetModeX %xMode320x240
    CASE 3 : xSetModeX %xMode320x400
    CASE 4 : xSetModeX %xMode360x480
  END SELECT

  xSetPal StrSeg(Pal), StrPtr(Pal), 0, LEN(Pal)\3

  '-- Create Virtual Screen ---------------------

  xGet 0, 0, 160, 160, Blank
  vScreen = Blank
  Active  = Blank

END SUB

'----------------------------------------------------------
'- Main
'----------------------------------------------------------

SUB Main

  '-- The Loop --------------------------

  DO

    DrawScreen
    DrawMan
    xDrawBitMap STRSEG(Active), STRPTR(Active)+4, 0, 0, 160, 160

    '-- Wait for a Keystroke --------------

    Keystroke First, Second

    IF First > 0 THEN
      IF First = 27 THEN
        EXIT DO
      END IF
    ELSE
      SELECT CASE Second

        CASE 71
          Man.Direction = 1
          X = Man.CurrentX - 1
          Y = Man.CurrentY - 1
          IF ASC(MID$(zMap(X), Y, 1)) = 254 THEN
            Man.CurrentX = X
            Man.CurrentY = Y
          END IF

        CASE 72
          Man.Direction = 1
          X = Man.CurrentX - 1
          Y = Man.CurrentY
          IF ASC(MID$(zMap(X), Y, 1)) = 254 THEN
            Man.CurrentX = X
          END IF

        CASE 73
          Man.Direction = 1
          X = Man.CurrentX - 1
          Y = Man.CurrentY + 1
          IF ASC(MID$(zMap(X), Y, 1)) = 254 THEN
            Man.CurrentX = X
            Man.CurrentY = Y
          END IF

        CASE 75
          Man.Direction = 4
          X = Man.CurrentX
          Y = Man.CurrentY - 1
          IF ASC(MID$(zMap(X), Y, 1)) = 254 THEN
            Man.CurrentY = Y
          END IF

        CASE 77
          Man.Direction = 2
          X = Man.CurrentX
          Y = Man.CurrentY + 1
          IF ASC(MID$(zMap(X), Y, 1)) = 254 THEN
            Man.CurrentY = Y
          END IF

        CASE 79
          Man.Direction = 3
          X = Man.CurrentX + 1
          Y = Man.CurrentY - 1
          IF ASC(MID$(zMap(X), Y, 1)) = 254 THEN
            Man.CurrentX = X
            Man.CurrentY = Y
          END IF

        CASE 80
          Man.Direction = 3
          X = Man.CurrentX + 1
          Y = Man.CurrentY
          IF ASC(MID$(zMap(X), Y, 1)) = 254 THEN
            Man.CurrentX = X
          END IF

        CASE 81
          Man.Direction = 3
          X = Man.CurrentX + 1
          Y = Man.CurrentY + 1
          IF ASC(MID$(zMap(X), Y, 1)) = 254 THEN
            Man.CurrentX = X
            Man.CurrentY = Y
          END IF

       END SELECT
    END IF

  LOOP

  '-- All Done ----------------------------------

  AllDone:

  xTextMode
  END

END SUB

'----------------------------------------------------------
'- DrawScreen: Draws walls onto vScreen
'              then copies vScreen to Active
'----------------------------------------------------------

SUB DrawScreen

  DIM DrawMore AS INTEGER, T       AS INTEGER,_
      StartX  AS INTEGER,  StartY  AS INTEGER,_
      EndX    AS INTEGER,  EndY    AS INTEGER,_
      OffsetX AS INTEGER,  OffsetY AS INTEGER,_
      A AS INTEGER, B AS INTEGER, C AS INTEGER,_
      I AS INTEGER
   
  '---Clear Virtual Screen

  vScreen = Blank

  '---Set Draw More

  DrawMore = 1

  '---Set Offsets

  OffsetX = 0
  OffsetY = 0

  '---Set StartX, StartY

  StartX = Man.CurrentX - 4
  EndX = Man.CurrentX + 4

  IF StartX < 1 THEN
     StartX = 1
     EndX = StartX + 8
  END IF

  IF EndX > 30 THEN
     EndX = 30
     StartX = EndX - 8
  END IF

  StartY = Man.CurrentY - 4
  EndY   = Man.CurrentY + 4

  IF StartY < 1 THEN
     StartY = 1
     EndY   = StartY + 8
  END IF

  IF EndY > 50 THEN
     EndY = 50
     StartY = EndY - 8
  END IF

  '---Do For...Next Loop to Draw the Virtual Screen

  FOR i = StartX TO EndX
    OffsetX = 0
    'IF MID$(zMap(i), Man.StartY, 10) = SPACE$(10) THEN EXIT FOR

    FOR t = StartY TO EndY
      A = ASC(MID$(zMap(i), t, 1))

      IF t = 1 THEN
        B = 0
      ELSE
        B = ASC(MID$(zMap(i), t - 1, 1))
      END IF

      IF t = 50 THEN
         C = 0
      ELSE
         C = ASC(MID$(zMap(i), t + 1, 1))
      END IF

      SELECT CASE A

        CASE 179
          vTPut VScreen, OffsetX, OffsetY, Walls

        CASE 196
          vTPutO VScreen, OffsetX, OffsetY, Walls, 130

        CASE 191
          IF C = 254 THEN vPutO  vScreen, OffsetX, OffsetY, Walls, 780
          vTPutO vScreen, OffsetX, OffsetY, Walls, 260

        CASE 192
          IF B = 254 THEN vPutO vScreen, OffsetX, OffsetY, Walls, 780
          vTPutO vScreen, OffsetX, OffsetY, Walls, 390

        CASE 217
          vTPutO vScreen, OffsetX, OffsetY, Walls, 520

        CASE 218
          vTPutO vScreen, OffsetX, OffsetY, Walls, 650

        CASE 254
          vPutO vScreen, OffsetX, OffsetY, Walls, 780

      END SELECT

      OffsetX = OffsetX + 16
    NEXT t
    OffsetY = OffsetY + 16
  NEXT i

  Active = vScreen

END SUB

'----------------------------------------------------------
'- DrawMan: Draws figure onto Active screen which already
'           contains the walls
'----------------------------------------------------------

SUB DrawMan

  DIM A  AS STRING, Offset AS INTEGER,_
      Di AS INTEGER,_
      StartX  AS INTEGER, StartY  AS INTEGER,_
      EndX    AS INTEGER, EndY    AS INTEGER,_
      OffsetX AS INTEGER, OffsetY AS INTEGER

  A$ = Man.Stage

  SELECT CASE A$

    CASE "1"
      Man.Stage = "2"
      Offset = 0

    CASE "2"
      Man.Stage = "3"
      Offset = 130

    CASE "3"
      Man.Stage = "1"
      Offset = 260

  END SELECT

  Di = Man.Direction

  SELECT CASE Di
    CASE 1 : Offset = Offset + 0
    CASE 2 : Offset = Offset + 390
    CASE 3 : Offset = Offset + 780
    CASE 4 : Offset = Offset + 1170
  END SELECT

  StartX = Man.CurrentX - 4
  EndX   = Man.CurrentX + 4
  IF StartX < 1 THEN
    StartX = 1
    EndX = StartX + 8
  END IF

  IF EndX > 30 THEN
    EndX = 30
    StartX = EndX - 8
  END IF

  StartY = Man.CurrentY - 4
  EndY = Man.CurrentY + 4

  IF StartY < 1 THEN
    StartY = 1
    EndY = StartY + 8
  END IF

  IF EndY > 50 THEN
    EndY = 50
    StartY = EndY - 8
  END IF

  OffsetY = 0
  OffsetX = 0

  FOR X = StartX TO EndX

    FOR Y = StartY TO EndY
      IF X = Man.CurrentX AND Y = Man.CurrentY THEN
        vTPutO Active, OffsetY, OffsetX, Figure, Offset
        EXIT FOR
      END IF
      OffsetY = OffsetY + 16
    NEXT Y

    OffsetX = OffsetX + 16
    OffsetY = 0
  NEXT X

END SUB

'----------------------------------------------------------
'- GetCharacter
'----------------------------------------------------------

SUB GetCharacter
  CLS
  LOCATE 1, 1
  PRINT "Select Type of Character:"

  LOCATE 3, 1
  PRINT "   A)  Ranger "
  PRINT "   B)  Thief  "
  PRINT "   C)  Paladin"
  PRINT "   D)  Mage   "

  i$ = UCASE$(INKEY$)
  WHILE i$ <> CHR$(27) AND (i$ < "A" OR i$ > "D")
    i$ = UCASE$(INKEY$)
  WEND
  IF i$ = CHR$(27) THEN GOTO AllDone

  SELECT CASE i$
    CASE "A" : Char$ = "Ranger"
    CASE "B" : Char$ = "Thief"
    CASE "C" : Char$ = "Paladin"
    CASE "D" : Char$ = "Mage"
  END SELECT
END SUB

'----------------------------------------------------------
'- GetMode
'----------------------------------------------------------

SUB GetMode

  CLS

  LOCATE 1, 1
  PRINT "Select Mode To Run:"

  LOCATE 3, 1
  PRINT "   1)  320x200"
  PRINT "   2)  320x240"
  PRINT "   3)  320x400"
  PRINT "   4)  360x480"

  WHILE NOT INSTAT : WEND
  I$ = INKEY$
  IF I$ = CHR$(27) THEN GOTO AllDone
  Mode$ = i$

END SUB

'----------------------------------------------------------
'- Keystroke
'----------------------------------------------------------

SUB Keystroke(First AS INTEGER, Second AS INTEGER)

  WHILE NOT INSTAT
  WEND

  I$ = INKEY$

  IF LEN(I$) = 1 THEN
    First = ASC(I$)
    Second = 0
  ELSE
    First = 0
    Second = ASC(RIGHT$(I$, 1))
  END IF

END SUB

'----------------------------------------------------------
'- xGet
'----------------------------------------------------------

SUB xGet (BYVAL X1 AS INTEGER, BYVAL Y1 AS INTEGER,_
          BYVAL X2 AS INTEGER, BYVAL Y2 AS INTEGER, Image AS STRING) PUBLIC

  DIM W  AS INTEGER, H  AS INTEGER

  IF X2 < X1 THEN
    W  = X1
    X1 = X2
    X2 = W
  END IF

  IF Y2 < Y1 THEN
    W  = Y1
    Y1 = Y2
    Y2 = W
  END IF

  W = X2 - X1
  H = Y2 - Y1

  Image = STRING$( (W * H)+4, 0 )
  xGetLbm x1, y1, x2, y2, Image

END SUB
